/* adal.h */

/*
 * constants
 */

#define noErr 0

#define CREATOR 'ADAL'

#define HISTORY_DB_TYPE 'HIST'
#define HISTORY_MAX_COUNT 14

#define PREF_VERSION 1

enum mode {
    MODE_NONE,
    MODE_APP,
    MODE_DA,
    MODE_BOTH,
    MODE_HISTORY,
    MODE_SWITCH_BACK
};

enum tap_again {
    TAPAGAIN_DO_NOTHING,
    TAPAGAIN_CLEAR_INPUT,
    TAPAGAIN_CLOSE_LAUNCHER,
    TAPAGAIN_OS_LAUNCHER
};

enum space {
    SPACE_NEXT_ITEM,
    SPACE_INPUT_SPACE
};

enum backspace {
    BACKSPACE_PREVIOUS_ITEM,
    BACKSPACE_CLEAR_CHAR,
    BACKSPACE_CLEAR_INPUT
};

enum page_key {
    PAGEKEY_SCROLL_LINE,
    PAGEKEY_SCROLL_PAGE
};

#define DEFAULT_BUTTON1 MODE_APP
#define DEFAULT_BUTTON2 MODE_DA
#define DEFAULT_BUTTON3 MODE_HISTORY
#define DEFAULT_BUTTON4 MODE_NONE

#define DEFAULT_TAP_AGAIN TAPAGAIN_CLOSE_LAUNCHER
#define DEFAULT_SPACE SPACE_NEXT_ITEM
#define DEFAULT_BACKSPACE BACKSPACE_CLEAR_INPUT
#define DEFAULT_PAGE_KEY PAGEKEY_SCROLL_PAGE

#define DEFAULT_INPUT_FEEDBACK true
#define DEFAULT_RECORD_HISTORY true
#define DEFAULT_HISTORY_COUNT 10

#define DEFAULT_STACK_SIZE 3000
#define STACK_UNIT 1000


/*
 * types
 */

struct history_entry {
    ULong type;
    ULong creator;
    char name[dmDBNameLength];
};

struct history_record {
    Word entry_size;
    Word entry_cnt;

#ifdef __GNUC__
    struct history_entry entries[0];
#else /* __MWERKS__ */
    struct history_entry entries[];
#endif
};

struct adal_prefs {
    Byte button1;
    Byte button2;
    Byte button3;
    Byte button4;

    Byte tap_again;
    Byte space;
    Byte backspace;
    Byte page_key;

    Byte input_feedback;
    Byte record_history;
    Word stack_size;

    Byte reserved[4];
};


/*
 * functions (!)
 */

static void get_prefs(struct adal_prefs *prefs)
{
    if (PrefGetAppPreferencesV10(
	    CREATOR, PREF_VERSION, prefs, sizeof(struct adal_prefs)) == false) {
	*((long *) &prefs->button1) =
		(long) DEFAULT_BUTTON1 << 24 |
		(long) DEFAULT_BUTTON2 << 16 |
		(long) DEFAULT_BUTTON3 << 8 |
		(long) DEFAULT_BUTTON4;

	*((long *) &prefs->tap_again) =
		(long) DEFAULT_TAP_AGAIN << 24 |
		(long) DEFAULT_SPACE << 16 |
		(long) DEFAULT_BACKSPACE << 8 |
		(long) DEFAULT_PAGE_KEY;

	*((long *) &prefs->input_feedback) =
		(long) DEFAULT_INPUT_FEEDBACK << 24 |
		(long) DEFAULT_RECORD_HISTORY << 16 |
		(long) DEFAULT_STACK_SIZE;

	*((long *) &prefs->reserved[0]) = 0;
    }
}
