/* adal_panel.c */

#include <Pilot.h>

#include "adal.h"
#include "adal_rcp.h"


/*
 * constants
 */

#define HACKMASTER_FORM_ID 9000

#define BAD_STACK_SIZE 0xff


/*
 * functions
 */

static VoidPtr GetObjectPtr(Word id)
{
    FormPtr form;

    form = FrmGetActiveForm();
    return FrmGetObjectPtr(form, FrmGetObjectIndex(form, id));
}

static void set_popup(Word popup_id, Byte value)
{
    ControlPtr popup;
    ListPtr list;
    CharPtr text;
    Word index;

    popup = GetObjectPtr(popup_id);
    list = GetObjectPtr(popup_id + 1);		/* list_id = popup_id + 1 */

    index = value;
    if (index < LstGetNumberOfItems(list)) {
	text = LstGetSelectionText(list, index);
    } else {
	text = "???";
	index = noListSelection;
    }

    CtlSetLabel(popup, text);
    LstSetSelection(list, index);
}

static void set_pref(Word list_id, Byte *value_ptr)
{
    ListPtr list;
    Word index;

    list = GetObjectPtr(list_id);
    index = LstGetSelection(list);
    if (index != noListSelection) {
	*value_ptr = index;
    }
}

static void clear_history()
{
    DmSearchStateType state;
    UInt card_no;
    LocalID db_id;

    if (DmGetNextDatabaseByTypeCreator(true, &state,
	    HISTORY_DB_TYPE, CREATOR, true, &card_no, &db_id) == noErr) {
	if (FrmAlert(CLEAR_HISTORY_ALERT_ID) == CLEAR_HISTORY_OK_INDEX) {
	    DmDeleteDatabase(card_no, db_id);
	}
    }
}

Boolean MyHandler(EventPtr event)
{
    Boolean handled = false;
    FormPtr form;
    struct adal_prefs prefs;
    Byte stack_size;

    switch (event->eType) {
    case frmOpenEvent:
	form = FrmGetActiveForm();

	get_prefs(&prefs);

	set_popup(PANEL_BUTTON1_POPUP_ID, prefs.button1);
	set_popup(PANEL_BUTTON2_POPUP_ID, prefs.button2);
	set_popup(PANEL_BUTTON3_POPUP_ID, prefs.button3);
	set_popup(PANEL_BUTTON4_POPUP_ID, prefs.button4);

	set_popup(PANEL_TAPAGAIN_POPUP_ID, prefs.tap_again);
	set_popup(PANEL_SPACE_POPUP_ID, prefs.space);
	set_popup(PANEL_BACKSPACE_POPUP_ID, prefs.backspace);
	set_popup(PANEL_PAGEKEY_POPUP_ID, prefs.page_key);

	set_popup(PANEL_FEEDBACK_POPUP_ID, prefs.input_feedback);
	set_popup(PANEL_HISTORY_POPUP_ID, prefs.record_history);

	stack_size = prefs.stack_size / STACK_UNIT;
	if (stack_size * STACK_UNIT != prefs.stack_size) {
	    stack_size = BAD_STACK_SIZE;
	}
	set_popup(PANEL_STACK_POPUP_ID, stack_size);

	FrmDrawForm(form);
	handled = true;
	break;

    case ctlSelectEvent:
	switch (event->data.ctlSelect.controlID) {
	case PANEL_OK_ID:
	    get_prefs(&prefs);

	    set_pref(PANEL_BUTTON1_LIST_ID, &prefs.button1);
	    set_pref(PANEL_BUTTON2_LIST_ID, &prefs.button2);
	    set_pref(PANEL_BUTTON3_LIST_ID, &prefs.button3);
	    set_pref(PANEL_BUTTON4_LIST_ID, &prefs.button4);

	    set_pref(PANEL_TAPAGAIN_LIST_ID, &prefs.tap_again);
	    set_pref(PANEL_SPACE_LIST_ID, &prefs.space);
	    set_pref(PANEL_BACKSPACE_LIST_ID, &prefs.backspace);
	    set_pref(PANEL_PAGEKEY_LIST_ID, &prefs.page_key);

	    set_pref(PANEL_FEEDBACK_LIST_ID, &prefs.input_feedback);
	    set_pref(PANEL_HISTORY_LIST_ID, &prefs.record_history);

	    stack_size = BAD_STACK_SIZE;
	    set_pref(PANEL_STACK_LIST_ID, &stack_size);
	    if (stack_size != BAD_STACK_SIZE) {
		prefs.stack_size = stack_size * STACK_UNIT;
	    }

	    PrefSetAppPreferencesV10(
		    CREATOR, PREF_VERSION, &prefs, sizeof(struct adal_prefs));

	    /* fall through */

	case PANEL_CANCEL_ID:
	    FrmGotoForm(HACKMASTER_FORM_ID);
	    handled = true;
	    break;

	case PANEL_DEFAULT_ID:
	    set_popup(PANEL_BUTTON1_POPUP_ID, DEFAULT_BUTTON1);
	    set_popup(PANEL_BUTTON2_POPUP_ID, DEFAULT_BUTTON2);
	    set_popup(PANEL_BUTTON3_POPUP_ID, DEFAULT_BUTTON3);
	    set_popup(PANEL_BUTTON4_POPUP_ID, DEFAULT_BUTTON4);

	    set_popup(PANEL_TAPAGAIN_POPUP_ID, DEFAULT_TAP_AGAIN);
	    set_popup(PANEL_SPACE_POPUP_ID, DEFAULT_SPACE);
	    set_popup(PANEL_BACKSPACE_POPUP_ID, DEFAULT_BACKSPACE);
	    set_popup(PANEL_PAGEKEY_POPUP_ID, DEFAULT_PAGE_KEY);

	    set_popup(PANEL_FEEDBACK_POPUP_ID, DEFAULT_INPUT_FEEDBACK);
	    set_popup(PANEL_HISTORY_POPUP_ID, DEFAULT_RECORD_HISTORY);
	    set_popup(PANEL_STACK_POPUP_ID, DEFAULT_STACK_SIZE / STACK_UNIT);

	    handled = true;
	    break;

	case PANEL_CLEAR_HISTORY_ID:
	    clear_history();
	    handled = true;
	    break;
	}
	break;

    default:		/* to shut up gcc -W ... */
	break;
    }

    return handled;
}
